@@ -7,6 +7,7 @@
 #include "vehicle.h"
 #include "json_p.h"
 #include "datatypes_p.h"
+#include "featureutil_p.h"
 #include "mergeutil_p.h"
 
 #include <QDebug>
@@ -15,6 +16,7 @@
 
 #include <limits>
 
+using namespace Qt::Literals::StringLiterals;
 using namespace KPublicTransport;
 
 namespace KPublicTransport {
@@ -27,10 +29,12 @@ public:
     float platformPositionEnd = -1.0;
     VehicleSection::Type type = VehicleSection::UnknownType;
     VehicleSection::Classes classes = VehicleSection::UnknownClass;
-    VehicleSection::Features features = VehicleSection::NoFeatures;
     int deckCount = 1;
     VehicleSection::Sides connectedSides = VehicleSection::Front | VehicleSection::Back;
     QString platformSectionName;
+    std::vector<Feature> sectionFeatures;
+    Disruption::Effect disruptionEffect = Disruption::NormalService;
+    Load::Category load = Load::Unknown;
 };
 
 class VehiclePrivate : public QSharedData
@@ -39,6 +43,7 @@ public:
     QString name;
     std::vector<VehicleSection> sections;
     Vehicle::Direction direction = Vehicle::UnknownDirection;
+    std::vector<Feature> features;
 };
 
 }
@@ -49,10 +54,92 @@ KPUBLICTRANSPORT_MAKE_PROPERTY(VehicleSection, float, platformPositionBegin, set
 KPUBLICTRANSPORT_MAKE_PROPERTY(VehicleSection, float, platformPositionEnd, setPlatformPositionEnd)
 KPUBLICTRANSPORT_MAKE_PROPERTY(VehicleSection, VehicleSection::Type, type, setType)
 KPUBLICTRANSPORT_MAKE_PROPERTY(VehicleSection, VehicleSection::Classes, classes, setClasses)
-KPUBLICTRANSPORT_MAKE_PROPERTY(VehicleSection, VehicleSection::Features, features, setFeatures)
 KPUBLICTRANSPORT_MAKE_PROPERTY(VehicleSection, int, deckCount, setDeckCount)
 KPUBLICTRANSPORT_MAKE_PROPERTY(VehicleSection, VehicleSection::Sides, connectedSides, setConnectedSides)
 KPUBLICTRANSPORT_MAKE_PROPERTY(VehicleSection, QString, platformSectionName, setPlatformSectionName)
+KPUBLICTRANSPORT_MAKE_PROPERTY(VehicleSection, KPublicTransport::Disruption::Effect, disruptionEffect, setDisruptionEffect)
+KPUBLICTRANSPORT_MAKE_PROPERTY(VehicleSection, KPublicTransport::Load::Category, load, setLoad)
+
+VehicleSection::Features VehicleSection::features() const
+{
+    struct {
+        KPublicTransport::Feature::Type type;
+        Feature legacyType;
+    } constexpr map[] = {
+        { KPublicTransport::Feature::AirConditioning, AirConditioning },
+        { KPublicTransport::Feature::Restaurant, Restaurant },
+        { KPublicTransport::Feature::ToddlerArea, ToddlerArea },
+        { KPublicTransport::Feature::WheelchairAccessible, WheelchairAccessible },
+        { KPublicTransport::Feature::SilentArea, SilentArea },
+        { KPublicTransport::Feature::BikeStorage, BikeStorage },
+    };
+
+    VehicleSection::Features features;
+    for (const auto &f : d->sectionFeatures) {
+        if (f.availability() == KPublicTransport::Feature::Unavailable) {
+            continue;
+        }
+        const auto it = std::find_if(std::begin(map), std::end(map), [&f](auto m) { return m.type == f.type(); });
+        if (it == std::end(map)) {
+            continue;
+        }
+        features |= (*it).legacyType;
+    }
+
+    return features;
+}
+
+[[nodiscard]] static constexpr KPublicTransport::Feature::Type fromLegacyFeature(int f)
+{
+    constexpr KPublicTransport::Feature::Type map[] = {
+        KPublicTransport::Feature::NoFeature,
+        KPublicTransport::Feature::AirConditioning,
+        KPublicTransport::Feature::Restaurant,
+        KPublicTransport::Feature::ToddlerArea,
+        KPublicTransport::Feature::WheelchairAccessible,
+        KPublicTransport::Feature::SilentArea,
+        KPublicTransport::Feature::BikeStorage,
+    };
+    return map[f];
+}
+
+void VehicleSection::setFeatures(Features features)
+{
+    d.detach();
+    if (features) {
+        std::vector<KPublicTransport::Feature> fs;
+        const auto me = QMetaEnum::fromType<VehicleSection::Features>();
+        for (int i = 0; i < me.keyCount(); ++i) {
+            if (const auto v = static_cast<VehicleSection::Feature>(me.value(i)); features & v) {
+                fs.emplace_back(fromLegacyFeature(i), KPublicTransport::Feature::Available);
+            }
+        }
+        FeatureUtil::set(d->sectionFeatures, std::move(fs));
+    } else {
+        d->sectionFeatures.clear();
+    }
+}
+
+const std::vector<KPublicTransport::Feature>& VehicleSection::sectionFeatures() const
+{
+    return d->sectionFeatures;
+}
+
+std::vector<KPublicTransport::Feature>&& VehicleSection::takeSectionFeatures()
+{
+    return std::move(d->sectionFeatures);
+}
+
+void VehicleSection::setSectionFeatures(std::vector<KPublicTransport::Feature> &&features)
+{
+    d.detach();
+    FeatureUtil::set(d->sectionFeatures, std::move(features));
+}
+
+KPublicTransport::Feature VehicleSection::feature(KPublicTransport::Feature::Type type) const
+{
+    return FeatureUtil::findByType(d->sectionFeatures, type);
+}
 
 VehicleSection VehicleSection::merge(const VehicleSection &lhs, const VehicleSection &rhs)
 {
@@ -72,12 +159,23 @@ VehicleSection VehicleSection::merge(const VehicleSection &lhs, const VehicleSec
     res.setDeckCount(std::max(lhs.deckCount(), rhs.deckCount()));
     res.setConnectedSides(lhs.connectedSides() & rhs.connectedSides());
     res.setPlatformSectionName(MergeUtil::mergeString(lhs.platformSectionName(), rhs.platformSectionName()));
+    res.setSectionFeatures(FeatureUtil::merge(lhs.sectionFeatures(), rhs.sectionFeatures()));
     return res;
 }
 
 QJsonObject VehicleSection::toJson(const VehicleSection &section)
 {
-    return Json::toJson(section);
+    auto obj = Json::toJson(section);
+    if (!section.d->sectionFeatures.empty()) {
+        obj.insert("features"_L1, KPublicTransport::Feature::toJson(section.d->sectionFeatures));
+    }
+    if (section.disruptionEffect() == Disruption::NormalService) {
+        obj.remove("disruptionEffect"_L1);
+    }
+    if (section.load() == Load::Unknown) {
+        obj.remove("load"_L1);
+    }
+    return obj;
 }
 
 QJsonArray VehicleSection::toJson(const std::vector<VehicleSection> &sections)
@@ -87,7 +185,15 @@ QJsonArray VehicleSection::toJson(const std::vector<VehicleSection> &sections)
 
 VehicleSection VehicleSection::fromJson(const QJsonObject &obj)
 {
-    return Json::fromJson<VehicleSection>(obj);
+    auto v = Json::fromJson<VehicleSection>(obj);
+    const auto fVal = obj.value("features"_L1);
+    if (fVal.isArray()) {
+        v.setSectionFeatures(KPublicTransport::Feature::fromJson(fVal.toArray()));
+    } else if (fVal.isString()) {
+        // backward compat for existing data
+        v.setFeatures(Json::flagsFromJson<Features>(fVal));
+    }
+    return v;
 }
 
 std::vector<VehicleSection> VehicleSection::fromJson(const QJsonArray &array)
@@ -100,8 +206,9 @@ QVariantList VehicleSection::featureList() const
     QVariantList l;
     const auto me = QMetaEnum::fromType<VehicleSection::Features>();
     for (int i = 0; i < me.keyCount(); ++i) {
-        if (features() & static_cast<VehicleSection::Feature>(1 << i))
+        if (features() & static_cast<VehicleSection::Feature>(1 << i)) {
             l.push_back(static_cast<VehicleSection::Feature>(1 << i));
+        }
     }
     return l;
 }
@@ -117,7 +224,7 @@ KPUBLICTRANSPORT_MAKE_PROPERTY(Vehicle, Vehicle::Direction, direction, setDirect
 
 bool Vehicle::isEmpty() const
 {
-    return d->name.isEmpty() && d->sections.empty() && d->direction == Vehicle::UnknownDirection;
+    return d->name.isEmpty() && d->sections.empty() && d->direction == Vehicle::UnknownDirection && d->features.empty();
 }
 
 const std::vector<VehicleSection>& Vehicle::sections() const
@@ -198,6 +305,8 @@ Vehicle Vehicle::merge(const Vehicle &lhs, const Vehicle &rhs)
         res.setSections(lhs.sections().size() < rhs.sections().size() ? rhs.sections() : lhs.sections());
     }
 
+    res.setFeatures(FeatureUtil::merge(lhs.features(), rhs.features()));
+
     return res;
 }
 
@@ -205,7 +314,10 @@ QJsonObject Vehicle::toJson(const Vehicle &vehicle)
 {
     auto obj = Json::toJson(vehicle);
     if (!vehicle.sections().empty()) {
-        obj.insert(QLatin1String("sections"), VehicleSection::toJson(vehicle.sections()));
+        obj.insert("sections"_L1, VehicleSection::toJson(vehicle.sections()));
+    }
+    if (!vehicle.features().empty()) {
+        obj.insert("features"_L1, Feature::toJson(vehicle.features()));
     }
     return obj;
 }
@@ -218,7 +330,8 @@ QJsonArray Vehicle::toJson(const std::vector<Vehicle> &vehicles)
 Vehicle Vehicle::fromJson(const QJsonObject &obj)
 {
     auto v = Json::fromJson<Vehicle>(obj);
-    v.setSections(VehicleSection::fromJson(obj.value(QLatin1String("sections")).toArray()));
+    v.setSections(VehicleSection::fromJson(obj.value("sections"_L1).toArray()));
+    v.setFeatures(Feature::fromJson(obj.value("features"_L1).toArray()));
     return v;
 }
 
@@ -237,4 +350,31 @@ bool Vehicle::hasPlatformSectionNames() const
     return std::none_of(d->sections.begin(), d->sections.end(), [](const auto &p) { return p.platformSectionName().isEmpty(); });
 }
 
+const std::vector<KPublicTransport::Feature>& Vehicle::features() const
+{
+    return d->features;
+}
+
+[[nodiscard]] std::vector<KPublicTransport::Feature>&& Vehicle::takeFeatures()
+{
+    return std::move(d->features);
+}
+
+void Vehicle::setFeatures(std::vector<KPublicTransport::Feature> &&features)
+{
+    d.detach();
+    d->features = std::move(features);
+}
+
+std::vector<KPublicTransport::Feature> Vehicle::combinedFeatures() const
+{
+    std::vector<KPublicTransport::Feature> features(d->features);
+    for (const auto &section : d->sections) {
+        for (const auto &feature : section.sectionFeatures()) {
+            FeatureUtil::aggregate(features, feature);
+        }
+    }
+    return features;
+}
+
 #include "moc_vehicle.cpp"
